@@ -39,8 +39,9 @@
 
 #include <string.h>
 #include <math.h>
+#include <X11/Xproto.h>
 
-#define MAX_HOME_VIEWS 4
+#define MAX_HOME_VIEWS 9
 
 #define HD_GCONF_DIR_VIEWS         "/apps/osso/hildon-desktop/views"
 #define HD_GCONF_KEY_VIEWS_ACTIVE  HD_GCONF_DIR_VIEWS "/active"
@@ -57,11 +58,16 @@ struct _HdHomeViewContainerPrivate
   guint previous_view;
   guint next_view;
 
-  int offset;
+  int offset;       // offset caused by used drag
+  int offset_anim;  // offset caused by animation
 
   HdHome *home;
   HdCompMgr *comp_mgr;
 
+  /* this is non-NULL if we have a global live background, common to all
+   * HdHomeViews */
+  MBWindowManagerClient *live_bg;
+
   /* animation */
   ClutterTimeline *timeline;
   int timeline_offset;
@@ -301,6 +307,141 @@ views_active_notify_func (GConfClient *client,
                                               FALSE);
 }
 
+static void
+remove_global_live_bg (HdHomeViewContainer *container)
+{
+  HdHomeViewContainerPrivate *priv = container->priv;
+  ClutterActor *actor;
+  MBWMCompMgrClutterClient *cclient;
+
+  if (!priv->live_bg) return;
+
+  cclient = MB_WM_COMP_MGR_CLUTTER_CLIENT (priv->live_bg->cm_client);
+  actor = mb_wm_comp_mgr_clutter_client_get_actor (cclient);
+
+  if (priv->live_bg->window->live_background == -1)
+    {
+      clutter_container_remove_actor (CLUTTER_CONTAINER (container), actor);
+      clutter_actor_hide (actor);
+    }
+  else if (priv->live_bg->window->live_background == -101)
+    {
+      ClutterActor *hfront = hd_home_get_front (priv->home);
+      clutter_container_remove_actor (CLUTTER_CONTAINER (hfront), actor);
+      clutter_actor_hide (actor);
+    }
+}
+
+void 
+hd_home_view_container_set_live_bg (HdHomeViewContainer *container,
+                                    MBWindowManagerClient *client)
+{
+  int view;
+  HdHomeView *hhview;
+  HdHomeViewContainerPrivate *priv = container->priv;
+  ClutterActor *actor;
+  MBWMCompMgrClutterClient *cclient;
+
+  view = client->window->live_background;
+  if (view > 0 && view <= MAX_HOME_VIEWS)
+    {
+      hhview = HD_HOME_VIEW (priv->views[view - 1]);
+      hd_home_view_set_live_bg (hhview, client, FALSE);
+    }
+  /* live-bg above applets */
+  else if (view > 100 && view <= 100 + MAX_HOME_VIEWS)
+    {
+      hhview = HD_HOME_VIEW (priv->views[view - 1 - 100]);
+      hd_home_view_set_live_bg (hhview, client, TRUE);
+    }
+  else if (view == 0)
+    {
+      /* remove live background */
+      int i;
+      if (priv->live_bg == client)
+        {
+          cclient = MB_WM_COMP_MGR_CLUTTER_CLIENT (client->cm_client);
+          actor = mb_wm_comp_mgr_clutter_client_get_actor (cclient);
+          clutter_container_remove_actor (CLUTTER_CONTAINER (container), actor);
+          clutter_actor_hide (actor);
+          priv->live_bg = NULL;
+
+          /* restore normal backgrounds */
+          for (i = 0; i < MAX_HOME_VIEWS; ++i)
+            {
+              hhview = HD_HOME_VIEW (priv->views[i]);
+              hd_home_view_load_background (hhview);
+            }
+        }
+      else
+        for (i = 0; i < MAX_HOME_VIEWS; ++i)
+          {
+            MBWindowManagerClient *c;
+            hhview = HD_HOME_VIEW (priv->views[i]);
+            c = hd_home_view_get_live_bg (hhview);
+            if (c && c == client)
+              {
+                hd_home_view_set_live_bg (hhview, NULL, FALSE);
+                /* restore normal background */
+                hd_home_view_load_background (hhview);
+              }
+          }
+    }
+  else if (view == -1)
+    {
+      /* use this live background on all desktops and without scrolling */
+      int i;
+      /* first remove all current live backgrounds */
+      for (i = 0; i < MAX_HOME_VIEWS; ++i)
+        {
+          hhview = HD_HOME_VIEW (priv->views[i]);
+          if (hd_home_view_get_live_bg (hhview))
+            hd_home_view_set_live_bg (hhview, NULL, FALSE);
+        }
+
+      if (priv->live_bg)
+        remove_global_live_bg (container);
+
+      cclient = MB_WM_COMP_MGR_CLUTTER_CLIENT (client->cm_client);
+      actor = mb_wm_comp_mgr_clutter_client_get_actor (cclient);
+      /* the actor is already parented to something, so reparent */
+      clutter_actor_reparent (actor, CLUTTER_ACTOR (container));
+      priv->live_bg = client;
+    }
+  else if (view == -101)
+    {
+      /* use this live background above applets on all desktops and without
+       * scrolling */
+      int i;
+      ClutterActor *hfront;
+
+      for (i = 0; i < MAX_HOME_VIEWS; ++i)
+        {
+          hhview = HD_HOME_VIEW (priv->views[i]);
+          if (hd_home_view_get_live_bg (hhview))
+            hd_home_view_set_live_bg (hhview, NULL, FALSE);
+        }
+
+      if (priv->live_bg)
+        remove_global_live_bg (container);
+
+      hfront = hd_home_get_front (priv->home);
+      cclient = MB_WM_COMP_MGR_CLUTTER_CLIENT (client->cm_client);
+      actor = mb_wm_comp_mgr_clutter_client_get_actor (cclient);
+      /* the actor is already parented to something, so reparent */
+      clutter_actor_reparent (actor, CLUTTER_ACTOR (hfront));
+      priv->live_bg = client;
+
+      /* restore normal backgrounds, FIXME: could be smarter by checking
+       * if the backgrounds are already there */
+      for (i = 0; i < MAX_HOME_VIEWS; ++i)
+        {
+          hhview = HD_HOME_VIEW (priv->views[i]);
+          hd_home_view_load_background (hhview);
+        }
+    }
+}
+
 static void
 hd_home_view_container_constructed (GObject *self)
 {
@@ -479,8 +620,10 @@ hd_home_view_container_allocate (ClutterActor          *self,
   width = box->x2 - box->x1;
   height = box->y2 - box->y1;
 
-  if (priv->previous_view != priv->current_view && priv->next_view != priv->current_view)
-    offset = CLUTTER_UNITS_FROM_INT(priv->offset);
+  if (priv->previous_view != priv->current_view
+      && priv->next_view != priv->current_view)
+    offset = CLUTTER_UNITS_FROM_INT(priv->offset) +
+             CLUTTER_UNITS_FROM_INT(priv->offset_anim);
 
   for (i = 0; i < MAX_HOME_VIEWS; i++)
     {
@@ -604,6 +747,12 @@ hd_home_view_container_get_current_view (HdHomeViewContainer *container)
   return priv->current_view;
 }
 
+MBWindowManagerClient *
+hd_home_view_container_get_live_bg (HdHomeViewContainer *container)
+{
+  return container->priv->live_bg;
+}
+
 void
 hd_home_view_container_set_current_view (HdHomeViewContainer *container,
                                          guint                current_view)
@@ -699,9 +848,6 @@ hd_home_view_container_set_offset (HdHomeViewContainer *container,
 
   priv = container->priv;
 
-  if (priv->timeline)
-    return;
-
   priv->offset = CLUTTER_UNITS_TO_INT(offset);
 
   clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
@@ -720,7 +866,7 @@ scroll_back_new_frame_cb (ClutterTimeline     *timeline,
   if (priv->animation_overshoot)
     amt = amt * -cos(amt*3.141592);
 
-  priv->offset = (int)((1-amt) * priv->timeline_offset);
+  priv->offset_anim = (int)((1-amt) * priv->timeline_offset);
   /* prod the blur control so it notices that the background has changed */
   hd_render_manager_blurred_changed();
 
@@ -755,13 +901,20 @@ hd_home_view_container_scroll_back (HdHomeViewContainer *container, gint velocit
 {
   HdHomeViewContainerPrivate *priv;
   guint width;
+  gint offset;
 
   g_return_if_fail (HD_IS_HOME_VIEW_CONTAINER (container));
 
   priv = container->priv;
 
-  if (priv->timeline)
-    return;
+  if (priv->timeline) {
+    clutter_timeline_stop(priv->timeline);
+    /* This will unref the timeline and switch desktops to what everything
+     * else is expecting */
+    scroll_back_completed_cb(priv->timeline, container);
+  }
+
+  offset = priv->offset + priv->offset_anim;
 
   clutter_actor_get_size (CLUTTER_ACTOR (container), &width, NULL);
 
@@ -775,7 +928,7 @@ hd_home_view_container_scroll_back (HdHomeViewContainer *container, gint velocit
     {
       /* Overshoot if we were going in one direction, but expect to
        * go in the other */
-      priv->animation_overshoot = (velocity>0) == (priv->offset>0);
+      priv->animation_overshoot = (velocity>0) == (offset>0);
       /* make sure velocity is within a sensible range, we don't want this
        * taking more than a second, or totally flicking past... */
       velocity = ABS(velocity);
@@ -787,18 +940,23 @@ hd_home_view_container_scroll_back (HdHomeViewContainer *container, gint velocit
    * the same as the drag velocity given in the argument (because the ease_out
    * function in scroll_back_new_frame_cb uses sin, and sin(x)==x near 0) */
   priv->timeline = clutter_timeline_new_for_duration
-                        (ABS (priv->offset) * 1570 / velocity);
+                        (ABS (offset) * 1570 / velocity);
 
   priv->frames = clutter_timeline_get_n_frames (priv->timeline);
-  priv->timeline_offset = priv->offset;
+  priv->timeline_offset = offset;
+  /* We reset this and use a separate offset (offset_anim) for animation so
+   * the user can still pan while we're animating */
+  priv->offset = 0;
 
   g_debug ("frames: %u, offset: %d",
-           priv->frames, priv->offset);
+           priv->frames, priv->timeline_offset);
 
   g_signal_connect (priv->timeline, "new-frame",
                     G_CALLBACK (scroll_back_new_frame_cb), container);
   g_signal_connect (priv->timeline, "completed",
                     G_CALLBACK (scroll_back_completed_cb), container);
+  /* Update first frame to stop flicker */
+  scroll_back_new_frame_cb(priv->timeline, 0, container);
 
   clutter_timeline_start (priv->timeline);
 }
@@ -813,9 +971,6 @@ hd_home_view_container_scroll_to_previous (HdHomeViewContainer *container, gint
 
   priv = container->priv;
 
-  if (priv->timeline)
-    return;
-
   clutter_actor_get_size (CLUTTER_ACTOR (container), &width, NULL);
 
   priv->offset -= width;
@@ -835,9 +990,6 @@ hd_home_view_container_scroll_to_next (HdHomeViewContainer *container, gint velo
 
   priv = container->priv;
 
-  if (priv->timeline)
-    return NULL;
-
   clutter_actor_get_size (CLUTTER_ACTOR (container), &width, NULL);
 
   priv->offset += width;
