@@ -47,7 +47,9 @@
 #include "hd-launcher.h"
 #include "hd-launcher-tree.h"
 #include "home/hd-render-manager.h"
+#include "home/hd-home-view-container.h"
 #include "hd-transition.h"
+#include "hd-wm.h"
 
 #undef  G_LOG_DOMAIN
 #define G_LOG_DOMAIN "hd-app-mgr"
@@ -205,6 +207,21 @@ G_DEFINE_TYPE (HdAppMgr, hd_app_mgr, G_TYPE_OBJECT);
 #define GCONF_SLIDE_OPEN_KEY     "/system/osso/af/slide-open"
 #define GCONF_DISABLE_CALLUI_DIR "/apps/osso/hildon-desktop"
 #define GCONF_DISABLE_CALLUI_KEY "/apps/osso/hildon-desktop/disable_phone_gesture"
+#define GCONF_KEY_ACTIONS_DIR "/apps/osso/hildon-desktop/key-actions"
+#define GCONF_VIEWS_CURRENT_DIR "/apps/osso/hildon-desktop/views"
+#define GCONF_VIEWS_CURRENT_KEY "/apps/osso/hildon-desktop/views/current"
+
+gboolean conf_enable_ctrl_backspace;
+gboolean conf_enable_preset_shift_ctrl;
+gboolean conf_enable_dbus_shift_ctrl;
+gboolean conf_enable_home_contacts_phone;
+gboolean conf_enable_launcher_navigator_accel;
+gboolean conf_enable_dbus_launcher_navigator;
+gboolean conf_default_launcher_positions;
+gboolean conf_dbus_shortcuts_use_fn;
+gboolean conf_dbus_ctrl_shortcuts;
+gint conf_ctrl_backspace_in_tasknav; 
+gboolean conf_disable_edit;
 
 /* Forward declarations */
 static void hd_app_mgr_dispose (GObject *gobject);
@@ -220,7 +237,6 @@ gboolean hd_app_mgr_hibernate (HdRunningApp *app);
 
 static gboolean hd_app_mgr_service_top (const gchar *service,
                                         const gchar *param);
-static gboolean  hd_app_mgr_execute (const gchar *exec, GPid *pid);
 
 void hd_app_mgr_prestartable     (HdRunningApp *app, gboolean prestartable);
 
@@ -264,7 +280,6 @@ static void hd_app_mgr_gconf_value_changed (GConfClient *client,
                                             guint cnxn_id,
                                             GConfEntry *entry,
                                             gpointer user_data);
-static void hd_app_mgr_mce_activate_accel  (void);
 static gboolean hd_app_mgr_show_callui_cb (gpointer data);
 static void hd_app_mgr_check_show_callui (void);
 
@@ -422,7 +437,7 @@ hd_app_mgr_init (HdAppMgr *self)
                                                     GCONF_DISABLE_CALLUI_KEY,
                                                     NULL);
       /* We don't call
-      hd_app_mgr_mce_activate_accel ();
+      hd_app_mgr_mce_activate_accel_if_needed ();
       here because hdrm is not ready yet. */
       gconf_client_add_dir (priv->gconf_client, GCONF_DISABLE_CALLUI_DIR,
                             GCONF_CLIENT_PRELOAD_NONE, NULL);
@@ -430,6 +445,40 @@ hd_app_mgr_init (HdAppMgr *self)
                                hd_app_mgr_gconf_value_changed,
                                (gpointer) self,
                                NULL, NULL);
+      gconf_client_notify_add (priv->gconf_client, GCONF_KEY_ACTIONS_DIR,
+                               hd_app_mgr_gconf_value_changed,
+                               (gpointer) self,
+                               NULL, NULL);
+      gconf_client_add_dir (priv->gconf_client, GCONF_VIEWS_CURRENT_DIR,
+                            GCONF_CLIENT_PRELOAD_NONE, NULL);
+      gconf_client_notify_add (priv->gconf_client, GCONF_VIEWS_CURRENT_KEY,
+                               hd_app_mgr_gconf_value_changed,
+                               (gpointer) self,
+                               NULL, NULL);
+      if(gconf_client_dir_exists(priv->gconf_client, GCONF_KEY_ACTIONS_DIR, NULL)) {
+	      conf_enable_ctrl_backspace = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/ctrl_backspace", NULL);
+	      conf_enable_preset_shift_ctrl = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/preset_shift_ctrl", NULL);
+	      conf_enable_dbus_shift_ctrl = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_shift_ctrl", NULL);
+	      conf_enable_home_contacts_phone = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/home_contacts_phone", NULL);
+	      conf_enable_launcher_navigator_accel = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/launcher_navigator_accel", NULL);
+	      conf_enable_dbus_launcher_navigator = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_launcher_navigator", NULL);
+	      conf_default_launcher_positions = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/default_launcher_positions", NULL);
+	      conf_ctrl_backspace_in_tasknav = gconf_client_get_int(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/ctrl_backspace_in_tasknav", NULL);
+	      conf_dbus_shortcuts_use_fn = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_shortcuts_use_fn", NULL);
+	      conf_dbus_ctrl_shortcuts = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_ctrl_shortcuts", NULL);
+	      conf_disable_edit = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/disable_edit", NULL);
+      }
     }
 
   /* Start memory limits. */
@@ -928,7 +977,7 @@ hd_app_mgr_start (HdRunningApp *app)
       if (exec)
         {
           GPid pid = 0;
-          result = hd_app_mgr_execute (exec, &pid);
+          result = hd_app_mgr_execute (exec, &pid, FALSE);
           if (result)
             {
               hd_running_app_set_pid (app, pid);
@@ -1348,8 +1397,8 @@ _hd_app_mgr_child_setup(gpointer user_data)
   }
 }
 
-static gboolean
-hd_app_mgr_execute (const gchar *exec, GPid *pid)
+gboolean
+hd_app_mgr_execute (const gchar *exec, GPid *pid, gboolean auto_reap)
 {
   gboolean res = FALSE;
   gchar *space = strchr (exec, ' ');
@@ -1382,7 +1431,7 @@ hd_app_mgr_execute (const gchar *exec, GPid *pid)
 
   res = g_spawn_async (NULL,
                        argv, NULL,
-                       G_SPAWN_DO_NOT_REAP_CHILD,
+                       auto_reap ? 0 : G_SPAWN_DO_NOT_REAP_CHILD,
                        _hd_app_mgr_child_setup, NULL,
                        pid,
                        NULL);
@@ -1574,7 +1623,7 @@ hd_app_mgr_hdrm_state_change (gpointer hdrm,
     }
 
   /* Also check if we should enable the accelerometer. */
-  hd_app_mgr_mce_activate_accel ();
+  hd_app_mgr_mce_activate_accel_if_needed (TRUE);
 }
 
 static void
@@ -1793,7 +1842,7 @@ hd_app_mgr_dbus_launch_app (HdAppMgr *self, const gchar *id)
 {
   HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (self);
   HdLauncherItem *item;
-  HdLauncherApp *app = NULL;
+  HdLauncherApp *app;
 
   if (!id || !strlen(id))
     { /* If no ID was specified, all we want to do is trigger the
@@ -1807,15 +1856,11 @@ hd_app_mgr_dbus_launch_app (HdAppMgr *self, const gchar *id)
     }
 
   item = hd_launcher_tree_find_item (priv->tree, id);
-
-  if (!item)
-    return FALSE;
-  if (hd_launcher_item_get_item_type (item) != HD_APPLICATION_LAUNCHER)
-    return FALSE;
-
-  app = HD_LAUNCHER_APP (item);
-
-  return hd_app_mgr_launch (app);
+  if (!item || hd_launcher_item_get_item_type (item) != HD_APPLICATION_LAUNCHER)
+    app = hd_launcher_tree_find_app_by_service (priv->tree, id);
+  else
+    app = HD_LAUNCHER_APP (item);
+  return app ? hd_app_mgr_launch (app) : FALSE;
 }
 
 gboolean
@@ -1841,13 +1886,15 @@ static gboolean
 _hd_app_mgr_should_show_callui ()
 {
   HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (hd_app_mgr_get ());
+  extern MBWindowManager *hd_mb_wm;
 
   if (STATE_SHOW_CALLUI (hd_render_manager_get_state ()) &&
       priv->portrait &&
       priv->unlocked &&
       priv->display_on &&
       priv->slide_closed &&
-      !priv->disable_callui)
+      !priv->disable_callui &&
+      !hd_wm_has_modal_blockers (hd_mb_wm))
     {
       return TRUE;
     }
@@ -1924,13 +1971,29 @@ _hd_app_mgr_dbus_check_value (DBusMessage *msg,
   return FALSE;
 }
 
+static void
+hd_app_mgr_update_portraitness(HdAppMgr *self)
+{
+  HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (self);
+  HdCompMgr *hmgr = hd_comp_mgr_get ();
+  // what about priv->unlocked/priv->display_on ?
+
+  if (!hmgr)
+    /* We're in some initialization state. */
+    return;
+  hd_comp_mgr_set_portrait_if_possible(hmgr,
+      priv->accel_enabled,
+      priv->portrait && priv->slide_closed);
+}
+
 static DBusHandlerResult
 hd_app_mgr_dbus_signal_handler (DBusConnection *conn,
                            DBusMessage *msg,
                            void *data)
 {
   gboolean changed = TRUE;
-  HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (HD_APP_MGR (data));
+  HdAppMgr *self = HD_APP_MGR (data);
+  HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (self);
 
   if (dbus_message_is_signal (msg,
                               LOWMEM_ON_SIGNAL_INTERFACE,
@@ -1978,6 +2041,8 @@ hd_app_mgr_dbus_signal_handler (DBusConnection *conn,
                                            MCE_ORIENTATION_PORTRAIT);
           if (priv->portrait)
             hd_app_mgr_check_show_callui ();
+
+          hd_app_mgr_update_portraitness(self);
         }
     }
 
@@ -1990,16 +2055,19 @@ hd_app_mgr_dbus_signal_handler (DBusConnection *conn,
 /* Activate the accelerometer when
  * - The user has activated rotate-to-callui.
  * - HDRM is in a state that shows callui.
+ * - We are showing an app, and all visible windows support portrait mode
  */
-static void
-hd_app_mgr_mce_activate_accel  ()
+void
+hd_app_mgr_mce_activate_accel_if_needed (gboolean update_portraitness)
 {
   HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (the_app_mgr);
   DBusConnection *conn = NULL;
   DBusMessage *msg = NULL;
 
-  gboolean activate = !priv->disable_callui &&
-                      STATE_SHOW_CALLUI (hd_render_manager_get_state ());
+  gboolean activate = (!priv->disable_callui &&
+                       STATE_SHOW_CALLUI (hd_render_manager_get_state ())) ||
+                      (STATE_IS_APP(hd_render_manager_get_state ()) &&
+                       hd_comp_mgr_can_be_portrait(hd_comp_mgr_get()));
   if (priv->accel_enabled == activate)
     return;
 
@@ -2054,6 +2122,9 @@ hd_app_mgr_mce_activate_accel  ()
   g_debug ("%s: %s", __FUNCTION__,
            activate? "enabled" : "disabled");
   priv->accel_enabled = activate;
+
+  if (update_portraitness)
+    hd_app_mgr_update_portraitness(the_app_mgr);
   return;
 }
 
@@ -2063,7 +2134,8 @@ hd_app_mgr_gconf_value_changed (GConfClient *client,
                                 GConfEntry *entry,
                                 gpointer user_data)
 {
-  HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (HD_APP_MGR (user_data));
+  HdAppMgr *self = HD_APP_MGR (user_data);
+  HdAppMgrPrivate *priv = HD_APP_MGR_GET_PRIVATE (self);
   GConfValue *gvalue;
   gboolean value = FALSE;
 
@@ -2074,9 +2146,45 @@ hd_app_mgr_gconf_value_changed (GConfClient *client,
   if (gvalue->type == GCONF_VALUE_BOOL)
     value = gconf_value_get_bool (gvalue);
 
+  if (!g_strcmp0 (gconf_entry_get_key (entry),
+                  GCONF_VIEWS_CURRENT_KEY)) {
+      HdHome * home =  hd_render_manager_get_home();
+
+      if(hd_home_view_container_get_current_view(hd_home_get_view_container(home)) != gconf_value_get_int(gvalue)-1)
+      	hd_home_view_container_set_current_view(hd_home_get_view_container(home), gconf_value_get_int(gvalue)-1);
+  }
+
   if (!g_strcmp0 (gconf_entry_get_key (entry),
                   GCONF_SLIDE_OPEN_KEY))
-    priv->slide_closed = !value;
+    {
+      priv->slide_closed = !value;
+
+      hd_app_mgr_update_portraitness(self);
+    }
+      if(gconf_client_dir_exists(priv->gconf_client, GCONF_KEY_ACTIONS_DIR, NULL)) {
+	      conf_enable_ctrl_backspace = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/ctrl_backspace", NULL);
+//	      conf_enable_preset_shift_ctrl = gconf_client_get_bool(priv->gconf_client, 
+//			      GCONF_KEY_ACTIONS_DIR "/preset_shift_ctrl", NULL);
+//	      conf_enable_dbus_shift_ctrl = gconf_client_get_bool(priv->gconf_client, 
+//			      GCONF_KEY_ACTIONS_DIR "/dbus_shift_ctrl", NULL);
+	      conf_enable_home_contacts_phone = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/home_contacts_phone", NULL);
+	      conf_enable_launcher_navigator_accel = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/launcher_navigator_accel", NULL);
+	      conf_enable_dbus_launcher_navigator = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_launcher_navigator", NULL);
+	      conf_default_launcher_positions = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/default_launcher_positions", NULL);
+	      conf_ctrl_backspace_in_tasknav = gconf_client_get_int(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/ctrl_backspace_in_tasknav", NULL);
+	      conf_dbus_shortcuts_use_fn = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_shortcuts_use_fn", NULL);
+	      conf_dbus_ctrl_shortcuts = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/dbus_ctrl_shortcuts", NULL);
+	      conf_disable_edit = gconf_client_get_bool(priv->gconf_client, 
+			      GCONF_KEY_ACTIONS_DIR "/disable_edit", NULL);
+      }
 
   if (!g_strcmp0 (gconf_entry_get_key (entry),
                   GCONF_DISABLE_CALLUI_KEY))
@@ -2084,7 +2192,7 @@ hd_app_mgr_gconf_value_changed (GConfClient *client,
       priv->disable_callui = value;
 
       /* Check if h-d needs to track the orientation. */
-      hd_app_mgr_mce_activate_accel ();
+      hd_app_mgr_mce_activate_accel_if_needed (TRUE);
     }
 
   return;
