@@ -48,9 +48,11 @@
 #include "hd-wm.h"
 #include "hd-theme.h"
 #include "hd-util.h"
+#include "hd-dbus.h"
 #include "hd-volume-profile.h"
 #include "launcher/hd-app-mgr.h"
 #include "home/hd-render-manager.h"
+#include "hd-transition.h"
 
 #ifndef DISABLE_A11Y
 #include "hildon-desktop-a11y.h"
@@ -61,6 +63,9 @@ enum {
   KEY_ACTION_TOGGLE_NON_COMP_MODE,
   KEY_ACTION_TAKE_SCREENSHOT,
   KEY_ACTION_XTERMINAL,
+  KEY_ACTION_TOGGLE_PORTRAITABLE,
+  KEY_ACTION_ROTATE,
+  KEY_ACTION_SEND_DBUS,
 };
 
 #ifdef MBWM_DEB_VERSION
@@ -69,6 +74,7 @@ asm(".string \"built with libmatchbox2 "MBWM_DEB_VERSION"\"");
 asm(".previous");
 #endif
 
+gboolean hd_debug_mode_set = FALSE;
 MBWindowManager *hd_mb_wm = NULL;
 static int hd_clutter_mutex_enabled = FALSE;
 static int hd_clutter_mutex_do_unlock_after_disabling = FALSE;
@@ -122,7 +128,7 @@ take_screenshot (void)
 {
   char *path, *filename;
   static gchar datestamp[255];
-  time_t secs;
+  static time_t secs = 0;
   struct tm *tm = NULL;
   GdkDrawable *window;
   int width, height;
@@ -135,11 +141,16 @@ take_screenshot (void)
     return;
   }
 
+  /* limit the rate of screenshots to avoid jamming HD when the key
+   * is pressed all the time */
+  if (time (NULL) - secs < 5)
+    return;
+
   path = g_strdup_printf ("%s/.images/Screenshots", getenv("MYDOCSDIR"));
   g_mkdir_with_parents (path, 0770);
 
   secs = time(NULL);
-  tm = localtime(&secs);  
+  tm = localtime(&secs);
   strftime (datestamp, 255, "%Y%m%d-%H%M%S", tm);
 
   filename = g_strdup_printf ("%s/Screenshot-%s.png",
@@ -186,6 +197,8 @@ theme_client_type_func (const char *type_name, void *user_data)
     return HdWmClientTypeAppMenu;
   else if (!strcmp (type_name, "home-applet"))
     return HdWmClientTypeHomeApplet;
+  else if (!strcmp (type_name, "status-menu"))
+      return HdWmClientTypeStatusMenu;
 
   return 0;
 }
@@ -200,6 +213,31 @@ theme_button_type_func (const char *type_name,
   return 0;
 }
 
+/* Toggle the portrait-capable flag on and off on the topmost window */
+static void
+toggle_portraitable(MBWindowManager   *wm)
+{
+  MBWindowManagerClient *c;
+  for (c=wm->stack_top;c;c=c->stacked_below)
+    {
+      MBWMClientType c_type = MB_WM_CLIENT_CLIENT_TYPE (c);
+      if (c_type == MBWMClientTypeDesktop)
+        break;
+      if (c_type == MBWMClientTypeApp)
+        {
+          /* actually set the portrait property to the opposite now */
+          gboolean new_supports = !hd_comp_mgr_client_supports_portrait(c);
+          guint value = new_supports ? 1 : 0;
+          mb_wm_util_async_trap_x_errors (wm->xdpy);
+          XChangeProperty(wm->xdpy, c->window->xwindow,
+                          wm->atoms[MBWM_ATOM_HILDON_PORTRAIT_MODE_SUPPORT],
+                          XA_CARDINAL, 32, PropModeReplace,
+                          (unsigned char *)&value, 1);
+          mb_wm_util_async_untrap_x_errors ();
+        }
+    }
+}
+
 static void
 key_binding_func (MBWindowManager   *wm,
 		  MBWMKeyBinding    *binding,
@@ -212,33 +250,90 @@ key_binding_func (MBWindowManager   *wm,
   switch (action)
     {
     case KEY_ACTION_TOGGLE_SWITCHER:
-      /* printf(" ### KEY_ACTION_TOGGLE_SWITCHER ###\n"); */
-      hd_render_manager_set_state (HDRM_STATE_TASK_NAV);
+      /* don't go to the switcher if we are showing a system-modal */
+      if (hd_render_manager_get_state () == HDRM_STATE_TASK_NAV ) {
+          switch(conf_ctrl_backspace_in_tasknav) {
+              case 1:
+                  hd_render_manager_set_state (HDRM_STATE_HOME);
+                  break;
+              case 2:
+                  hd_render_manager_set_state (HDRM_STATE_LAUNCHER);
+                  break;
+              case 3:
+                  hd_task_navigator_activate(-2, -2, 0);
+                  break;
+              case 4:
+                  hd_task_navigator_activate(-1, -2, 0);
+                  break;
+              case 5:
+                  in_alt_tab = TRUE;
+                  hd_task_navigator_rotate_thumbs();
+                  break;
+  
+              case 0:
+              default:
+                  break;
+          }
+      } else { 
+		  if (!hd_wm_has_modal_blockers (hd_mb_wm))
+			hd_render_manager_set_state (HDRM_STATE_TASK_NAV);
+            if(conf_ctrl_backspace_in_tasknav==5) { 
+                in_alt_tab = TRUE;
+                    hd_task_navigator_sort_thumbs();
+                hd_task_navigator_rotate_thumbs();
+            }
+      }
       break;
     case KEY_ACTION_TOGGLE_NON_COMP_MODE:
       /* printf(" ### KEY_ACTION_TOGGLE_NON_COMP_MODE ###\n"); */
       if (hd_render_manager_get_state () == HDRM_STATE_NON_COMPOSITED)
         hd_render_manager_set_state (HDRM_STATE_APP);
-      else
-        hd_render_manager_set_state (HDRM_STATE_NON_COMPOSITED);
+      else if (hd_render_manager_get_state () == HDRM_STATE_NON_COMP_PORT)
+        hd_render_manager_set_state (HDRM_STATE_APP_PORTRAIT);
+      else if (hd_render_manager_get_state () == HDRM_STATE_APP)
+        {
+          hd_render_manager_set_state (HDRM_STATE_NON_COMPOSITED);
+          /* render manager does not unredirect non-fullscreen apps,
+           * so do it here */
+          hd_comp_mgr_unredirect_topmost_client (hd_mb_wm, TRUE);
+        }
+      else if (hd_render_manager_get_state () == HDRM_STATE_APP_PORTRAIT)
+        {
+          hd_render_manager_set_state (HDRM_STATE_NON_COMP_PORT);
+          hd_comp_mgr_unredirect_topmost_client (hd_mb_wm, TRUE);
+        }
       break;
     case KEY_ACTION_TAKE_SCREENSHOT:
         take_screenshot();
 	break;
     case KEY_ACTION_XTERMINAL:
       {
-	pid_t fork(void);
-	int execl(const char *path, const char *arg, ...);
-
-	pid_t havechild=fork();
-	if (!havechild)
-	  execl("/usr/bin/osso-xterm", 
-		"/usr/bin/osso-xterm", NULL);
-	/* This is a hack - don't care about the kid */
+        GPid pid;
+        if (hd_app_mgr_execute ("/usr/bin/osso-xterm", &pid, TRUE))
+          g_spawn_close_pid (pid);
+        break;
       }
-      break;
+    case KEY_ACTION_TOGGLE_PORTRAITABLE:
+        toggle_portraitable(wm);
+        break;
+    case KEY_ACTION_ROTATE:
+        hd_transition_rotate_screen (wm, !hd_comp_mgr_is_portrait());
+        break;
     }
 }
+static void
+key_binding_func_key (MBWindowManager   *wm,
+		  MBWMKeyBinding    *binding,
+		  void              *userdata)
+{
+  int action;
+  char s[32];
+
+  action = (int)(userdata);
+  sprintf(s,"%i",action);
+
+  hd_dbus_send_event (s);
+}
 
 static ClutterX11FilterReturn
 clutter_x11_event_filter (XEvent *xev, ClutterEvent *cev, gpointer data)
@@ -265,6 +360,11 @@ dump_debug_info_when_idle (gpointer unused)
 static void
 dump_debug_info_sighand (int unused)
 {
+  if (hd_debug_mode_set)
+    hd_debug_mode_set = FALSE;
+  else
+    hd_debug_mode_set = TRUE;
+
   g_idle_add (dump_debug_info_when_idle, GINT_TO_POINTER (0));
   signal(SIGUSR1, dump_debug_info_sighand);
 }
@@ -478,6 +578,7 @@ main (int argc, char **argv)
   Display * dpy = NULL;
   MBWindowManager *wm;
   HdAppMgr *app_mgr;
+  char keys1[32], c; 
 
   signal (SIGUSR1, dump_debug_info_sighand);
   signal (SIGHUP,  relaunch);
@@ -508,6 +609,9 @@ main (int argc, char **argv)
   mb_wm_theme_set_custom_button_type_func (theme_button_type_func, NULL);
 
   gtk_init (&argc, &argv);
+  /* Initialise the async error handler. Do it after gtk is inited, or gtk
+   * will grab the handler for itself */
+  mb_wm_util_async_x_error_init();
 
   hd_mutex_init ();
 
@@ -524,6 +628,9 @@ main (int argc, char **argv)
   /* Disable mipmapping of text, as it is seldom scaled down and this
    * saves us memory/bandwidth/update speed */
   clutter_set_use_mipmapped_text(FALSE);
+  /* Use software-based selection, which is much faster on SGX than rendering
+   * with 'GL and reading back */
+  clutter_set_software_selection(TRUE);
 
 #ifndef DISABLE_A11Y
   hildon_desktop_a11y_init ();
@@ -552,26 +659,118 @@ main (int argc, char **argv)
   mb_wm_init (wm);
   g_assert (mb_wm_comp_mgr_enabled (wm->comp_mgr));
 
-  mb_wm_keys_binding_add_with_spec (wm,
-				    "<ctrl>BackSpace",
-				    key_binding_func,
-				    NULL,
-				    (void*)KEY_ACTION_TOGGLE_SWITCHER);
-  mb_wm_keys_binding_add_with_spec (wm,
-				    "<shift><ctrl>x",
-				    key_binding_func,
-				    NULL,
-				    (void*)KEY_ACTION_XTERMINAL);
-  mb_wm_keys_binding_add_with_spec (wm,
-				    "<shift><ctrl>n",
-				    key_binding_func,
-				    NULL,
-				    (void*)KEY_ACTION_TOGGLE_NON_COMP_MODE);
-  mb_wm_keys_binding_add_with_spec (wm,
-				    "<shift><ctrl>p",
-				    key_binding_func,
-				    NULL,
-				    (void*)KEY_ACTION_TAKE_SCREENSHOT);
+  if(conf_enable_ctrl_backspace) {
+	  mb_wm_keys_binding_add_with_spec (wm,
+						"<ctrl>BackSpace",
+						key_binding_func,
+						NULL,
+						(void*)KEY_ACTION_TOGGLE_SWITCHER);
+  }
+  if(conf_enable_preset_shift_ctrl) {
+	  mb_wm_keys_binding_add_with_spec (wm,
+						"<shift><ctrl>x",
+						key_binding_func,
+						NULL,
+						(void*)KEY_ACTION_XTERMINAL);
+	  mb_wm_keys_binding_add_with_spec (wm,
+						"<shift><ctrl>n",
+						key_binding_func,
+						NULL,
+						(void*)KEY_ACTION_TOGGLE_NON_COMP_MODE);
+	  mb_wm_keys_binding_add_with_spec (wm,
+						"<shift><ctrl>p",
+						key_binding_func,
+						NULL,
+						(void*)KEY_ACTION_TAKE_SCREENSHOT);
+	  mb_wm_keys_binding_add_with_spec (wm,
+										  "<shift><ctrl>r",
+										  key_binding_func,
+										  NULL,
+										  (void*)KEY_ACTION_TOGGLE_PORTRAITABLE);
+	  mb_wm_keys_binding_add_with_spec (wm, /* mod5 == Fn */
+										  "<shift><ctrl><mod5>l",
+										  key_binding_func,
+										  NULL,
+										  (void*)KEY_ACTION_ROTATE);
+  }
+
+  if(conf_enable_dbus_shift_ctrl) {
+      if(conf_dbus_shortcuts_use_fn) {
+          mb_wm_keys_binding_add_with_spec (wm,
+                           "<ctrl><mod5>Space",
+                           key_binding_func_key,
+                           NULL,
+                           (void*)(192+32));
+          mb_wm_keys_binding_add_with_spec (wm,
+                           "<ctrl><mod5>comma",
+                           key_binding_func_key,
+                           NULL,
+                           (void*)(192+33));
+          mb_wm_keys_binding_add_with_spec (wm,
+                           "<ctrl><mod5>period",
+                           key_binding_func_key,
+                           NULL,
+                           (void*)(192+34));
+      } else {
+          mb_wm_keys_binding_add_with_spec (wm,
+                           "<shift><ctrl>Space",
+                           key_binding_func_key,
+                           NULL,
+                           (void*)(192+32));
+          mb_wm_keys_binding_add_with_spec (wm,
+                           "<shift><ctrl>comma",
+                           key_binding_func_key,
+                           NULL,
+                           (void*)(192+33));
+          mb_wm_keys_binding_add_with_spec (wm,
+                           "<shift><ctrl>period",
+                           key_binding_func_key,
+                           NULL,
+                           (void*)(192+34));
+      }
+	  if(conf_dbus_ctrl_shortcuts) {
+		  mb_wm_keys_binding_add_with_spec (wm,
+							"<ctrl>F7",
+							key_binding_func_key,
+							NULL,
+							(void*)247);
+		  mb_wm_keys_binding_add_with_spec (wm,
+							"<ctrl>F8",
+							key_binding_func_key,
+							NULL,
+							(void*)248);
+		  mb_wm_keys_binding_add_with_spec (wm,
+						   "<ctrl>Space",
+						   key_binding_func_key,
+						   NULL,
+						   (void*)(192+36));
+		  mb_wm_keys_binding_add_with_spec (wm,
+						   "<ctrl>comma",
+						   key_binding_func_key,
+						   NULL,
+						   (void*)(192+37));
+		  mb_wm_keys_binding_add_with_spec (wm,
+						   "<ctrl>period",
+						   key_binding_func_key,
+						   NULL,
+						   (void*)(192+38));
+	  }
+
+	  if(conf_dbus_shortcuts_use_fn) {
+		  strcpy(keys1,"<ctrl><mod5>a");
+	  } else {
+		  strcpy(keys1,"<shift><ctrl>a");
+	  }
+	  for(c='a';c<='z';c++) if(!conf_enable_preset_shift_ctrl || conf_dbus_shortcuts_use_fn || ((c!='n') && (c!='p') && (c!='x') && (c!='h'))){
+		  keys1[strlen(keys1)-1]=c;
+		  mb_wm_keys_binding_add_with_spec (wm,
+						keys1,
+						key_binding_func_key,
+						NULL,
+						(void*)(192+c-'a'+1));
+	  }
+  }
+
 
   clutter_x11_add_filter (clutter_x11_event_filter, wm);
 
