@@ -52,9 +52,6 @@ hd_atoms_init (Display * xdpy, Atom * atoms)
 
     "WM_WINDOW_ROLE",
 
-    "_HILDON_PORTRAIT_MODE_SUPPORT",
-    "_HILDON_PORTRAIT_MODE_REQUEST",
-
     "_HILDON_DO_NOT_DISTURB",
     "_HILDON_DO_NOT_DISTURB_OVERRIDE",
 
