@@ -29,7 +29,10 @@
 #include <clutter/clutter.h>
 #include <math.h>
 
-#define TIDY_FINGER_SCROLL_INITIAL_SCROLLBAR_DELAY (1000)
+#define TIDY_FINGER_SCROLL_INITIAL_SCROLLBAR_DELAY (2000)
+#define TIDY_FINGER_SCROLL_FADE_SCROLLBAR_IN_TIME (250)
+#define TIDY_FINGER_SCROLL_FADE_SCROLLBAR_OUT_TIME (500)
+#define TIDY_FINGER_SCROLL_DRAG_TRASHOLD (25)
 
 G_DEFINE_TYPE (TidyFingerScroll, tidy_finger_scroll, TIDY_TYPE_SCROLL_VIEW)
 
@@ -49,6 +52,11 @@ struct _TidyFingerScrollPrivate
   /* Scroll mode */
   TidyFingerScrollMode mode;
 
+  /* @move tells whether TIDY_FINGER_SCROLL_DRAG_TRASHOLD has been exceeded.
+   * @first_x and @first_y are the coordinates of the first touch. */
+  gboolean               move;
+  ClutterFixed           first_x, first_y;
+
   GArray                *motion_buffer;
   guint                  last_motion;
 
@@ -62,7 +70,8 @@ struct _TidyFingerScrollPrivate
   ClutterFixed           bounce_back_speed_rate;
 
   /* Variables to fade in/out scroll-bars */
-  ClutterEffectTemplate *template;
+  ClutterEffectTemplate *template_in;
+  ClutterEffectTemplate *template_out;
   ClutterTimeline       *hscroll_timeline;
   ClutterTimeline       *vscroll_timeline;
 
@@ -154,10 +163,15 @@ tidy_finger_scroll_dispose (GObject *object)
       priv->vscroll_timeline = NULL;
     }
 
-  if (priv->template)
+  if (priv->template_in)
     {
-      g_object_unref (priv->template);
-      priv->template = NULL;
+      g_object_unref (priv->template_in);
+      priv->template_in = NULL;
+    }
+  if (priv->template_out)
+    {
+      g_object_unref (priv->template_out);
+      priv->template_out = NULL;
     }
 
   G_OBJECT_CLASS (tidy_finger_scroll_parent_class)->dispose (object);
@@ -228,9 +242,24 @@ motion_event_cb (ClutterActor *actor,
           dy = CLUTTER_UNITS_TO_FIXED(motion->y - y) +
                tidy_adjustment_get_valuex (vadjust);
 
-          tidy_adjustment_set_valuex (hadjust, dx);
-          tidy_adjustment_set_valuex (vadjust, dy);
-          clutter_actor_queue_redraw( actor );
+          /* Has the drag treshold been reached? */
+          if (!priv->move)
+            {
+                ClutterFixed d1 = x - priv->first_x;
+                ClutterFixed d2 = y - priv->first_y;
+                priv->move = clutter_qmulx (d1, d1) + clutter_qmulx (d2, d2)
+                  >= CLUTTER_INT_TO_FIXED (TIDY_FINGER_SCROLL_DRAG_TRASHOLD
+                                           * TIDY_FINGER_SCROLL_DRAG_TRASHOLD);
+            }
+
+          /* If not, do everything as if it had (we already did) except
+           * for adjusting @child's position. */
+          if (priv->move)
+            {
+              tidy_adjustment_set_valuex (hadjust, dx);
+              tidy_adjustment_set_valuex (vadjust, dy);
+              clutter_actor_queue_redraw( actor );
+            }
         }
 
       priv->last_motion ++;
@@ -302,7 +331,7 @@ show_scrollbars (TidyFingerScroll *scroll, gboolean show)
   /* Create new ones */
   if (!CLUTTER_ACTOR_IS_REACTIVE (hscroll))
     priv->hscroll_timeline = clutter_effect_fade (
-                               priv->template,
+                               show ? priv->template_in : priv->template_out,
                                hscroll,
                                show ? 0xFF : 0x00,
                                (ClutterEffectCompleteFunc)hfade_complete_cb,
@@ -310,7 +339,7 @@ show_scrollbars (TidyFingerScroll *scroll, gboolean show)
 
   if (!CLUTTER_ACTOR_IS_REACTIVE (vscroll))
     priv->vscroll_timeline = clutter_effect_fade (
-                               priv->template,
+                               show ? priv->template_in : priv->template_out,
                                vscroll,
                                show ? 0xFF : 0x00,
                                (ClutterEffectCompleteFunc)vfade_complete_cb,
@@ -755,6 +784,13 @@ captured_event_cb (ClutterActor     *actor,
         {
           g_get_current_time (&motion->time);
 
+          /* Save the coordinates of the first touch to be able to determine
+           * whether we've exceeded the drag treshold when processing motion
+           * events.  Until then don't move @child. */
+          priv->move = FALSE;
+          priv->first_x = motion->x;
+          priv->first_y = motion->y;
+
           if (priv->deceleration_timeline)
             {
               clutter_timeline_stop (priv->deceleration_timeline);
@@ -838,7 +874,8 @@ static void
 tidy_finger_scroll_init (TidyFingerScroll *self)
 {
   ClutterActor *scrollbar;
-  ClutterTimeline *effect_timeline;
+  ClutterTimeline *effect_timeline_in;
+  ClutterTimeline *effect_timeline_out;
   TidyFingerScrollPrivate *priv = self->priv = FINGER_SCROLL_PRIVATE (self);
   ClutterFixed qn;
   guint i;
@@ -846,7 +883,7 @@ tidy_finger_scroll_init (TidyFingerScroll *self)
   priv->motion_buffer = g_array_sized_new (FALSE, TRUE,
                                            sizeof (TidyFingerScrollMotion), 3);
   g_array_set_size (priv->motion_buffer, 3);
-  priv->decel_rate = CLUTTER_FLOAT_TO_FIXED(0.99f);
+  priv->decel_rate = CLUTTER_FLOAT_TO_FIXED(0.90f);
   priv->bouncing_decel_rate = CLUTTER_FLOAT_TO_FIXED (0.7f);
 
   /*
@@ -890,9 +927,14 @@ tidy_finger_scroll_init (TidyFingerScroll *self)
   g_signal_connect (scrollbar, "notify::reactive",
                     G_CALLBACK (vscroll_notify_reactive_cb), self);
 
-  effect_timeline = clutter_timeline_new_for_duration (250);
-  priv->template = clutter_effect_template_new (effect_timeline,
-                                                CLUTTER_ALPHA_RAMP_INC);
+  effect_timeline_in = clutter_timeline_new_for_duration
+                            (TIDY_FINGER_SCROLL_FADE_SCROLLBAR_IN_TIME);
+  priv->template_in = clutter_effect_template_new (effect_timeline_in,
+                                                   CLUTTER_ALPHA_RAMP_INC);
+  effect_timeline_out = clutter_timeline_new_for_duration
+                            (TIDY_FINGER_SCROLL_FADE_SCROLLBAR_OUT_TIME);
+  priv->template_out = clutter_effect_template_new (effect_timeline_out,
+                                                    CLUTTER_ALPHA_RAMP_INC);
 }
 
 ClutterActor *
